/********************************************

ATTINY45
LDR
PIN
SERVOMOTOR

connections:
SERVOPIN 	PB1
LED 		PB3
LDR 		PB2

using
ADC for LDR
interrupt
PWM for servo

2011 www.contrechoc.com
used to make a butterfly flap its wings
in the head of a bride, during her wedding!

feel free to modify and exploit!

*********************************************/


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define PWMDEFAULT 0x00 // default PWM value at start up for all channels

#define set(x) |= (1<<x)
#define clr(x) &=~(1<<x)
#define inv(x) ^=(1<<x)

#define SERVOPIN PB1

#define LED_PORT PORTB
#define LED_DDR DDRB

void delay_ms(uint16_t ms);
void doubledelay_ms(uint16_t ms);
void init();

unsigned char compare;
unsigned char g_dir;
volatile unsigned char flapFactor;
volatile unsigned char flapCount;
volatile unsigned char compbuff;
short raw, oldRaw;

unsigned short totalCount;

//these are the limits of the wings clapping, can be adjusted
unsigned char upper = 48;
unsigned char lower = 15;

volatile short timerCounter = 0;

float dim = 1;

short abs(short val);


short abs(short val){
if (val < 0 ) return -val;
return val;
}

void adc_init()
{
	/* internal pull-ups interfere with the ADC. disable the
	 * pull-up on the pin if it's being used for ADC. either
	 * writing 0 to the port register or setting it to output
	 * should be enough to disable pull-ups. */
	PORTB &= ~_BV(PB2);
	//DDRB = 0x00;
	DDRB &= ~_BV(PB2);
	/* unless otherwise configured, arduinos use the internal Vcc
	 * reference. MUX 0x0f samples the ground (0.0V). */
	ADMUX = 0x0D;//B1101 = GND attiny45 page 139
	/*
	 * Enable the ADC system, use 128 as the clock divider on a 16MHz
	 * arduino (ADC needs a 50 - 200kHz clock) and start a sample. the
	 * AVR needs to do some set-up the first time the ADC is used; this
	 * first, discarded, sample primes the system for later use.
	 */
	ADCSRA |= _BV(ADEN) | _BV(ADPS1) | _BV(ADPS0) | _BV(ADSC);
	/* wait for the ADC to return a sample */
	loop_until_bit_is_clear(ADCSRA, ADSC);
}

unsigned short adc_read(unsigned char pin)
{
	unsigned char l, h, r;

	r = (ADMUX & 0xf0) | (pin & 0x0f);
	ADMUX = r; /* select the input channel */
	ADCSRA |= _BV(ADSC);
	loop_until_bit_is_clear(ADCSRA, ADSC);

	/* must read the low ADC byte before the high ADC byte */
	l = ADCL;
	h = ADCH;

	return ((unsigned short)h << 8) | l;
}

int main() {
	

	adc_init();//initialize the LDR input

   	init();

  	DDRB |= (1<<PB3);//set led OUTPUT
  
  	g_dir = 5;
	flapFactor = 1;
	compbuff = (upper + lower)/2;

  	for(;;) {
			raw = adc_read(1);//reading change in light value
		    delay_ms(10);
			short flapDiff = abs(raw - oldRaw);
			if ( flapDiff >  250 ) {//starting on light difference 
				flapFactor = 1;
				flapCount = 3;
				PORTB &=~(1<<PB3);
			}
			else 
			{
				if ( flapCount < 1 )
					flapFactor = 0;//stopping
			}
			oldRaw = raw;//saving LDR value for comparison
  			}
}



void doubledelay_ms(uint16_t ms) {
	unsigned char c = 0;

  	while (ms--) {
 		c = 2;
  		while (c--) {
		_delay_ms(1);
		}
  	}
}
 
void delay_ms(uint16_t ms) {
  while (ms) {
    _delay_ms(1);
    ms--;
  }
}

void init(void) {
  // set the direction of the ports

  DDRB set(SERVOPIN);

  //CLKPR = (1 << CLKPCE); // enable clock prescaler update
  //CLKPR = 0; // set clock to maximum (= crystal)

  // initialise all channels
  //compare = lower; // set default PWM values

  TIFR = (1 << TOV0); // clear interrupt flag
  TIMSK = (1 << TOIE0); // enable overflow interrupt
  TCCR0B = (1 << CS00); // start timer, no prescale


//timer 
  TCCR1 = (1 << CTC1) | (1 << CS10)| (1 << CS12); //WGNM12 bit3 of tccr1b timer control register
  OCR1A = (uint16_t)0xff;//output compare register
  TIMSK |= 1 << OCIE1A;   // Output Compare Interrupt Enable (timer 1, OCR1A)
 
  sei();
}

SIGNAL( TIMER1_COMPA_vect  ){

	timerCounter++;	

	if ( (timerCounter % 0x0020) == 0 ){
		if (compbuff > upper) g_dir = -1;
		if (compbuff < lower) { g_dir = 1; flapCount--; }
		compbuff += g_dir*flapFactor;
	}

	if ( (timerCounter % 0x0F00) == 0 ) PORTB |= (1<<PB3);
	if ( (timerCounter % 0x0F05) == 0 ) PORTB &=~(1<<PB3);

//flapping once in a while
	if ( (timerCounter % 0xFFFF) == 0 ) //for flapping once in a while
		{
			totalCount = 0;
			flapCount = 3;
			flapFactor = 1;
		}
	 
}


ISR (TIM0_OVF_vect) { //PWM timer for the server
  static unsigned char pinlevelB=(1 << PB1);
  static unsigned char softcount=0xFF;
if ( flapFactor != 0 ){

if ( pinlevelB == 0 ) PORTB &= ~(1 << PB1); else PORTB |= (1 << PB1); 

  if(++softcount == 0){ // increment modulo 256 counter and update
                                // the compare values only when counter = 0.
    compare= compbuff;
    pinlevelB = 1; // set PB1 pins high
 
  }

  // clear port pin on compare match (executed on next interrupt)
  if(compare == softcount)  pinlevelB = 0;// set PB1 pins low

  }

}
